/*
 * A registry of filesystem object providers.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <map>
#include <mutex>

#include <mem/types.h>



/* Forward declaration. */

class Provider;



/* Mapping type from file identifiers to providers. */

typedef std::map<fileid_t, Provider *> ProviderMapping;
typedef std::pair<fileid_t, Provider *> ProviderMappingEntry;



/* A registry of filesystem object providers. */

class ProviderRegistry
{
protected:
    ProviderMapping _providers;
    std::mutex _lock;

    /* Filesystem object access. */

    Provider *get(fileid_t fileid);

    void remove(fileid_t fileid, Provider *obj);

    void set(fileid_t fileid, Provider *obj);

public:
    /* Methods for resources. */

    void detach(fileid_t fileid, Provider *mapper);
};

// vim: tabstop=4 expandtab shiftwidth=4
